#
# Construct the main user interface screen
#

package require Tk
package require Tkhtml

::mailroom::initialize

if {[::mailroom::db onecolumn {SELECT COUNT(*) FROM var}] < 5} {
  wm withdraw .
  ::mailroom::config option config buckets -value "+inbox spam trash"
  ::mailroom::config popup 1
  wm state . normal
}

# Create our resources
image create bitmap arrow_up -data "
#define triangleUp7x4_width 7
#define triangleUp7x4_height 4
static unsigned char triangleUp7x4_bits[] = {
   0x08, 0x1c, 0x3e, 0x7f};
"
image create bitmap arrow_dn -data "
#define triangleDn7x4_width 7
#define triangleDn7x4_height 4
static unsigned char triangleDn7x4_bits[] = {
   0x7f, 0x3e, 0x1c, 0x08};
"

font create TkFixedFontBold {*}[font actual TkFixedFont] -weight bold
font create TkTextFontBold {*}[font actual TkTextFont] -weight bold

set_main_title

set g [::mailroom::config option config geometry]
if {$g!=""} {
  wm geometry . $g
}

option add *Menu.tearOff 0

menu .mb -type menubar
switch $tcl_platform(platform) {
  unix {
    if {$::tcl_platform(os) == "Darwin"} {
      . configure -menu .mb
      menu .mb.apple
      
      .mb add cascade -menu .mb.apple
      .mb.apple add command -label "About $::appTitle v$::mailroom::appVersion" -command help_about
  
      proc ::tk::mac::ShowPreferences {} { ::mailroom::config _popup }
    } else {
      pack .mb -side top -fill x
    }
  }
  default {
    . configure -menu .mb
  }
}

.mb add cascade -label File -underline 0 -menu .mb.file
menu .mb.file
.mb.file add command -label {Save As...} -command {export selected}
.mb.file add command -label {Export Visible...} -command {export visible}
.mb.file add command -label {Export All...} -command {export all}
.mb.file add command -label {Import From File...} -command {::mailroom::gimport import_from_file}
.mb.file add command -label {Import From DB...} -command import_from_db
if {$::tcl_platform(os) != "Darwin"} {
  .mb.file add command -label {Configure...} -command { ::mailroom::config popup }
}
.mb.file add cascade -label Empty -underline 0 -menu .mb.file.empty
.mb.file add separator
.mb.file add command -label Exit -command {quit_app} -underline 1

menu .mb.file.empty
.mb.file.empty add command -label {Empty Trash} -underline 6 \
  -command {
    ::mailroom::message delete_by_bucket {trash}
    message_rebuild_catalog
  }
.mb.file.empty add command -label {Empty Spam} -underline 6 \
  -command {
    ::mailroom::message delete_by_bucket {spam}
    message_rebuild_catalog
  }
.mb.file.empty add separator
.mb.file.empty add command -label {Empty All} -underline 6 \
  -command {
    ::mailroom::message delete_by_bucket {trash spam}
    message_rebuild_catalog
  }

.mb add cascade -label Edit -underline 0 -menu .mb.edit
menu .mb.edit
.mb.edit add command -label Cut -underline 2 -accel Ctrl-X \
   -state disabled
.mb.edit add command -label Copy -underline 0 -state disabled \
   -accel Ctrl-C
.mb.edit add command -label Paste -underline 0 -state disabled \
   -accel Ctrl-V
.mb.edit add command -label Delete -underline 0 -state disabled \
   -accel Del

.mb add cascade -label Help -underline 0 -menu .mb.help
menu .mb.help
.mb.help add command -label About -command help_about


# Setup buckets
set buckets {}
array set show {}
foreach bucket [split \
  [::mailroom::db onecolumn {SELECT value FROM var WHERE name1='config' AND name2='buckets'}]] \
{
  set active 0
  if {[string index $bucket 0]=="+"} {
    set active 1
    set bucket [string range $bucket 1 end]
  }
  
  lappend buckets $bucket $active
}

# Button Bar 1
ttk::frame .bb1
pack .bb1 -side top -fill x
proc setbtn {w args} {
  eval ttk::button $w $args
  pack $w -side left -fill y
}
ttk::label .bb1.label -text {Set Bucket:}
pack .bb1.label -side left

proc assign_bucket {newbucket args} {
  set move 0
  
  foreach {a v} $args {
    switch -- $a {
      -move { set move $v }
    }
  }
  
  switch -- $::toptext {
    .nb.p0.t {
      if {$move} {
        set move [.nb.p1.t next $::selected]
        if {$move == {}} {
          # Indicate that we've hit the end of the line and we are backing up
          bell
          
          set move [.nb.p1.t prev $::selected]
          if {$move == {}} {
            # Two beeps would indicate that we have no more messages to navigate.
            bell
            
            # Just go back to an empty catalog.
            show_catalog
            
            set move 0
          }
        }
      }

      ::mailroom::message set_bucket $::selected $newbucket
      if {$::show($newbucket) == 0} {
        .nb.p1.t delete $::selected
      }
    }
    
    .nb.p1.t {
      set optArgs {}
      set reply [tk_dialog .confirmMassMove "Confirm Mass Move" \
        "Are you sure you wish to move all visible messages into the '$newbucket' bucket?" \
        questhead 0 Yes {Yes and Mark Read} No]
      if {$reply==2} {
        return
      } elseif {$reply==1} {
        set optArgs {-markRead 1}
      }
      
      ::mailroom::db transaction {
        foreach msgid [.nb.p1.t children {}] {
          ::mailroom::message set_bucket $msgid $newbucket {*}$optArgs
        }
      }
      
      # If we are not showing the new bucket, remove all these messages.
      if {$::show($newbucket) == 0} {
        .nb.p1.t delete [.nb.p1.t children {}]
      }
      
      set move 0
    }
  }
  
  if {$move} {
    set ::selected $move
    message_display $::selected
    show_msg
  }
  
  updateMessageStatus
}

proc updateMessageStatus {} {
  switch -- $::toptext {
    .nb.p0.t {
      set ::status "Message [expr {1+[.nb.p1.t index $::selected]}] of [llength [.nb.p1.t children {}]]"
      update
    }
    .nb.p1.t {
      set ::status "[llength [.nb.p1.t children {}]] matching messages"
      update
    }
  }
}

foreach {name def} $buckets {
  set text [string totitle $name]
  setbtn .bb1.$name -text $text -width [expr 1 + [string length $text]]
  bind .bb1.$name <ButtonRelease-1> [list assign_bucket $name -move 1]
  bind .bb1.$name <Shift-ButtonRelease-1> [list assign_bucket $name -move 0]
}

# Button Bar 2
ttk::frame .bb2
pack .bb2 -side top -fill x

ttk::frame .bb  ;# The button bar
pack .bb -side top -fill x
proc bbbutton {w txt args} {
  eval ttk::button $w -text $txt -width [expr [string length $txt] + 1] $args
  pack $w -side left -fill y
}
proc statusUpdate {msg} {
  set ::fetchStatus $msg
  update
}
bbbutton .bb.fetch Fetch      -command {::mailroom::fetch begin statusUpdate; message_rebuild_catalog}
bbbutton .bb.new New          -command compose_begin
bbbutton .bb.cat List         -command show_catalog
bbbutton .bb.msg Msg          -command message_display
bbbutton .bb.html Html        -command message_display_html
bbbutton .bb.raw Raw          -command message_display_raw
bbbutton .bb.mime Mime        -command message_display_mime
bbbutton .bb.nx Next          -command {message_next; message_display; updateMessageStatus}
bbbutton .bb.prev Prev        -command {message_previous; message_display; updateMessageStatus}
bbbutton .bb.reply1 Reply     -command {compose_reply [compose_begin] 0}
bbbutton .bb.reply2 Reply-All -command {compose_reply [compose_begin] 1}
bbbutton .bb.fwd Forward      -command {compose_forward [compose_begin]}
#bbbutton .bb.cut Cut          -command {cut_text $toptext}
#bbbutton .bb.copy Copy        -command {copy_text $toptext}
#bbbutton .bb.paste Paste      -command {paste_text $toptext}
bbbutton .bb.log Log          -command show_log

# Message View Container
ttk::frame .nb -width 500 -height 500
pack .nb -side top -fill both -expand 1

# Status Bar
set ::status "Ready..."
set ::fetchStatus ""
ttk::frame .sb
ttk::label .sb.status -textvariable ::status -relief sunken
ttk::label .sb.fetch -textvariable ::fetchStatus -relief sunken -width 40
pack .sb.status -side left -fill x -expand 1 -ipadx 2 -ipady 2
pack .sb.fetch -side left -ipadx 2 -ipady 2 -padx {2 0}
pack .sb -side bottom -fill x -after .nb

# Message Detail View
ttk::frame .nb.p0
place .nb.p0 -x 0 -y 0 -relwidth 1 -relheight 1
text .nb.p0.t -yscrollcommand {.nb.p0.vsb set} -cursor arrow -width 10 -height 2 -exportselection 1
.nb.p0.t tag config header -foreground #005500 -background #efefef -font TkFixedFontBold
.nb.p0.t tag config q1 -foreground #ff0000
.nb.p0.t tag config q2 -foreground #cc0000
.nb.p0.t tag config q3 -foreground #990000
.nb.p0.t tag config q4 -foreground #660000
.nb.p0.t tag config q5 -foreground #330000
.nb.p0.t tag config url -foreground blue
.nb.p0.t tag bind url <1> {url_clicked %W %x %y}

proc url_clicked {W  x y} {
  set idx [$W index @$x,$y]
  set urlIdx [$W tag prevrange url $idx]
  set url [$W get {*}[split $urlIdx]]
  
  if {$url!=""} {
    launch_external_browser $url
  }
}

.nb.p0.t tag config mimesave -foreground blue
.nb.p0.t tag config mimeview -foreground blue
.nb.p0.t tag bind mimesave <1> {mime_extract save %x %y}
.nb.p0.t tag bind mimeview <1> {mime_extract view %x %y}
ttk::scrollbar .nb.p0.vsb -orient vertical -command {.nb.p0.t yview}

grid .nb.p0.t -row 0 -column 0 -sticky nsew
grid .nb.p0.vsb -row 0 -column 1 -sticky ns
grid rowconfigure .nb.p0 0 -weight 1
grid columnconfigure .nb.p0 0 -weight 1

proc quick_reply {{allflag 0}} {
  global selected

  ::mailroom::db eval {
    SELECT data, datetime(received) AS 'when', bucket
      FROM msg JOIN blob ON msgid=blobid
     WHERE msgid=$selected
  } break
  if {![info exists data]} return
  set m [mime::initialize -string $data]
  set hdrlist [mime::getheader $m]
  set hdr(received) {}
  set body [::mailroom::message::_body $m]
  set addHdrs {}
  mime::finalize $m
    
  foreach {name value} $hdrlist {
    set hdr([string tolower $name]) $value
  }

  if {[info exists hdr(from)]} {
    set from [::mailroom::user::getProper [lindex $hdr(from) 0]]
  } else {
    set from *anonymous*
  }

  set selRange [.nb.p0.t tag ranges sel]
  
  if {$selRange!=""} {
    set body [.nb.p0.t get {*}$selRange]
  }
  
  regsub -all \n \n[string map {\r {}} [string trim $body]] "\n> " body
  
  set reply [.nb.p0.qr.t get 1.0 {end-1 char}]
  
  if {$::qrSignature} {
    append reply [::mailroom::db onecolumn {
        SELECT value FROM var WHERE name1='config' AND name2='signature'
    }]
  }
  
  if {$::qrTopReply} {
    set body "$reply\n\n$from wrote:\n$body"
  } else {
    set body "$from wrote:\n$body\n\n$reply"
  }

  set msg [mime::initialize -canonical text/plain -string $body]

  if {[info exists hdr(message-id)]} {
    lappend addHdrs In-Reply-To [lindex $hdr(message-id) 0]
    lappend addHdrs References [lindex $hdr(message-id) 0]
  }
  if {[info exists hdr(reply-to)]} {
    foreach addr $hdr(reply-to) {
      lappend addHdrs To [::mailroom::user::getProper $addr]
    }
  } elseif {[info exists hdr(from)]} {
    foreach addr $hdr(from) {
      lappend addHdrs To [::mailroom::user::getProper $addr]
    }
  }
  if {$allflag} {
    if {[info exists hdr(to)]} {
      foreach addr $hdr(to) {
        lappend addHdrs To [::mailroom::user::getProper $addr]
      }
    }
    if {[info exists hdr(cc)]} {
      foreach addr $hdr(cc) {
        lappend addHdrs Cc [::mailroom::user::getProper $addr]
      }
    }
  }
  if {[info exists hdr(subject)]} {
    set subj "Re: [::mailroom::message::_subject_fix [lindex $hdr(subject) 0] 1]"
  } else {
    set subj "Re: (no subject)"
  }
  lappend addHdrs Subject $subj
  
  ::mailroom::message send -mime $msg -headers $addHdrs -bucket $bucket -statusCmd statusUpdate

  mime::finalize $msg
}

set ::qrTopReply 0
set ::qrSignature 0

ttk::frame .nb.p0.qr
ttk::frame .nb.p0.qr.tb
ttk::button .nb.p0.qr.tb.reply -text "Quick Reply" -command {quick_reply}
ttk::button .nb.p0.qr.tb.reply_all -text "Quick Reply All" -command {quick_reply 1}
ttk::checkbutton .nb.p0.qr.tb.top_reply -text "Top Reply" -variable ::qrTopReply
ttk::checkbutton .nb.p0.qr.tb.signature -text "Include Signature" -variable ::qrSignature
pack .nb.p0.qr.tb.reply .nb.p0.qr.tb.reply_all .nb.p0.qr.tb.top_reply .nb.p0.qr.tb.signature \
  -side left -padx 2
grid .nb.p0.qr.tb -row 0 -column 0 -columnspan 2 -sticky we -pady 2

text .nb.p0.qr.t -yscrollcommand {.nb.p0.qr.vsb set} -width 10 -height 7 -wrap word
ttk::scrollbar .nb.p0.qr.vsb -orient vertical -command {.nb.p0.qr.t yview}
grid .nb.p0.qr.t -row 1 -column 0 -sticky nsew
grid .nb.p0.qr.vsb -row 1 -column 1 -sticky ns

grid rowconfigure .nb.p0.qr 1 -weight 1
grid columnconfigure .nb.p0.qr 0 -weight 1
grid .nb.p0.qr -row 1 -column 0 -columnspan 2 -sticky nsew

# Message List View
set ::orderby date
set ::orderdir ASC

proc sort_mail {W column} {
  set currentSort [$W heading $column -image]
  
  foreach c [$W cget -columns] {
    $W heading $c -image {}
  }
  
  if {$currentSort=="arrow_up"} {
    set newSort arrow_dn
    set ::orderdir ASC
  } else {
    set newSort arrow_up
    set ::orderdir DESC
  }
  
  $W heading $column -image $newSort
  set ::orderby $column
  
  message_rebuild_catalog_when_idle
}

ttk::frame .nb.p1
place .nb.p1 -x 0 -y 0 -relwidth 1 -relheight 1

ttk::labelframe .nb.p1.filter -text "Message List Filter" -padding {5 2 5 2}

# Message List Button Bar 1
ttk::frame .nb.p1.filter.bb1
ttk::label .nb.p1.filter.bb1.label -text {Text:}
pack .nb.p1.filter.bb1.label -side left
set keywords ""
ttk::entry .nb.p1.filter.bb1.e -textvariable keywords
pack .nb.p1.filter.bb1.e -side left -fill x -expand 1
bind .nb.p1.filter.bb1.e <Return> {.nb.p1.filter.bb1.go invoke; focus -force .nb.p1.filter.bb1.e}
bind .nb.p1.filter.bb1.e <Shift-Return> {.nb.p1.filter.bb1.goall invoke; focus -force .nb.p1.filter.bb1.e}
bind .nb.p1.filter.bb1.e <Escape> {.nb.p1.filter.bb1.clr invoke; focus -force .nb.p1.filter.bb1.e}
ttk::button .nb.p1.filter.bb1.go -text Find -width 5 -command {message_rebuild_catalog}
ttk::button .nb.p1.filter.bb1.goall -text {Find All} -width 9 -command {message_rebuild_catalog 1}
ttk::button .nb.p1.filter.bb1.clr -text Clear -width 6 -command {set ::keywords {}; message_rebuild_catalog}
pack .nb.p1.filter.bb1.go .nb.p1.filter.bb1.goall .nb.p1.filter.bb1.clr -side left
pack .nb.p1.filter.bb1 -side top -fill x

# Message List View button bar 2
proc only_one_bucket {name} {
  foreach {n d} $::buckets {
    if {$name!=$n} {
      set ::show($n) 0
    }
  }
  
  # a bit backward, but we ensure it's off then the normal button handler
  # takes over and sets it to on.
  set ::show($name) 0
}

ttk::frame .nb.p1.filter.bb2
set limit {30 Days}
ttk::label .nb.p1.filter.bb2.limit_lbl -text {Limit:}
ttk::combobox .nb.p1.filter.bb2.limit -textvariable ::limit -height 9 \
  -width 10 -values {
    {48 Hours} {7 Days} {14 Days} {30 Days} 
    {60 Days} {90 Days} {6 Months} {12 Months}
    None
  }
pack .nb.p1.filter.bb2.limit_lbl -side left
pack .nb.p1.filter.bb2.limit -side left

set ::showRead 1
set ::showMine 0

ttk::label .nb.p1.filter.bb2.include_lbl -text {Include:}
ttk::checkbutton .nb.p1.filter.bb2.read -text Read -var ::showRead -command {}
ttk::checkbutton .nb.p1.filter.bb2.mine -text Mine -var ::showMine -command {}
pack .nb.p1.filter.bb2.include_lbl -side left -fill y -padx {10 0}
pack .nb.p1.filter.bb2.read .nb.p1.filter.bb2.mine -side left -fill y

proc ckbtn {w name var} {
  ttk::checkbutton $w -text [string totitle $name] -var $var
  bind $w <Shift-ButtonRelease-1> [list only_one_bucket $name]
  pack $w -side left -fill y
}
ttk::label .nb.p1.filter.bb2.label -text {Show:}
pack .nb.p1.filter.bb2.label -side left -fill y -padx {10 0}

foreach {name def} $buckets {
  set show($name) $def
  ckbtn .nb.p1.filter.bb2.$name $name show($name)
}
pack .nb.p1.filter.bb2 -side top -fill x

ttk::treeview .nb.p1.t -yscrollcommand {.nb.p1.vsb set} -show headings \
  -columns {date subject from size snippet} -displaycolumns {date subject from size}
.nb.p1.t tag configure unread -font TkTextFontBold ;#-foreground blue
.nb.p1.t heading #1 -text Date -image arrow_dn -command {sort_mail .nb.p1.t date}
.nb.p1.t column #1 -minwidth 60 -width 130 -stretch 0
.nb.p1.t heading #2 -text Subject -command {sort_mail .nb.p1.t subject}
.nb.p1.t column #2 -minwidth 150 -width 200 -stretch 1
.nb.p1.t heading #3 -text From -command {sort_mail .nb.p1.t from}
.nb.p1.t column #3 -minwidth 100 -width 130 -stretch 1
.nb.p1.t heading #4 -text Size -anchor e -command {sort_mail .nb.p1.t size}
.nb.p1.t column #4 -minwidth 40 -width 60 -stretch 0 -anchor e

ttk::scrollbar .nb.p1.vsb -orient vertical -command {.nb.p1.t yview}

grid .nb.p1.filter    -row 0 -column 0 -sticky ew -columnspan 2 -padx 8 -pady 4
grid .nb.p1.t         -row 1 -column 0 -sticky nsew
grid .nb.p1.vsb       -row 1 -column 1 -sticky ns

grid rowconfigure .nb.p1 1 -weight 1
grid columnconfigure .nb.p1 0 -weight 1

bind .nb.p1.t <<TreeviewSelect>> { message_display [%W selection] }
bind .nb.p1.t <Motion> {message_mouseover %x %y %X %Y}
bind .nb.p1.t <Leave> {message_mouseover -1 0 0 0}

# Message HTML View
ttk::frame .nb.p2
place .nb.p2 -x 0 -y 0 -relwidth 1 -relheight 1
html .nb.p2.t -yscrollcommand {.nb.p2.vsb set} -width 10 -height 2

pack .nb.p2.t -side left -fill both -expand 1
ttk::scrollbar .nb.p2.vsb -orient vertical -command {.nb.p2.t yview}
pack .nb.p2.vsb -side left -fill y

# Log View
ttk::frame .nb.log
place .nb.log -x 0 -y 0 -relwidth 1 -relheight 1

text .nb.log.t -yscrollcommand {.nb.log.vsb set} -width 10 -height 2
ttk::scrollbar .nb.log.vsb -orient vertical -command {.nb.log.t yview}

ttk::frame .nb.log.bb
ttk::button .nb.log.bb.clear -text Clear -command {.nb.log.t delete 1.0 end}
ttk::button .nb.log.bb.save -text Save -command {
  set fn [tk_getSaveFile -filetypes {{{Text Files} *.txt}} -initialfile "mailroom_log.txt"]
  if {$fn!=""} {
    set fh [open $fn w]
    puts $fh [.nb.log.t get 1.0 "end-1 char"]
    close $fh
  }
}
pack .nb.log.bb.clear .nb.log.bb.save -side left -padx 2 -pady 2

grid .nb.log.t -row 0 -column 0 -sticky nsew
grid .nb.log.vsb -row 0 -column 1 -sticky ns
grid .nb.log.bb -row 1 -column 0 -columnspan 2 -sticky nw
grid rowconfigure .nb.log 0 -weight 1
grid columnconfigure .nb.log 0 -weight 1

# Make the message text box appear on top
#
proc show_msg {} {
  global toptext
  set toptext .nb.p0.t
  raise .nb.p0
  .bb.cat config -state normal
  .bb.msg config -state normal
  .bb.html config -state normal
  .bb.raw config -state normal
  .bb.mime config -state normal
  .bb.nx config -state normal
  .bb.prev config -state normal
  .bb.reply1 config -state normal
  .bb.reply2 config -state normal
  .bb.fwd config -state normal

  foreach {name def} $::buckets {
    .bb1.$name configure -state normal
  }
  
  updateMessageStatus
}


# Make the html view appear on top
#
proc show_html {} {
  global toptext
  set toptext .nb.p2.t
  raise .nb.p2
  .bb.cat config -state normal
  .bb.msg config -state normal
  .bb.html config -state normal
  .bb.raw config -state normal
  .bb.mime config -state normal
  .bb.nx config -state normal
  .bb.prev config -state normal
  .bb.reply1 config -state normal
  .bb.reply2 config -state normal
  .bb.fwd config -state normal
  .bb.log config -state normal

  foreach {name def} $::buckets {
    .bb1.$name configure -state normal
  }
  
  updateMessageStatus
}

proc show_log {} {
  global toptext
  set toptext .nb.log.t
  raise .nb.log
  .bb.cat config -state normal
  .bb.msg config -state disabled
  .bb.html config -state disabled
  .bb.raw config -state disabled
  .bb.mime config -state disabled
  .bb.nx config -state disabled
  .bb.prev config -state disabled
  .bb.reply1 config -state disabled
  .bb.reply2 config -state disabled
  .bb.fwd config -state disabled
  .bb.log config -state disabled

  foreach {name def} $::buckets {
    .bb1.$name configure -state disabled
  }
  
  updateMessageStatus
}

# Make the catalog text box appear on top
#
proc show_catalog {} {
  global toptext
  set toptext .nb.p1.t
  raise .nb.p1
  
  .bb.cat config -state disabled
  .bb.msg config -state disabled
  .bb.html config -state disabled
  .bb.raw config -state disabled
  .bb.mime config -state disabled
  .bb.nx config -state disabled
  .bb.prev config -state disabled
  .bb.reply1 config -state disabled
  .bb.reply2 config -state disabled
  .bb.fwd config -state disabled
  .bb.log config -state normal
  
  foreach {name def} $::buckets {
    .bb1.$name configure -state normal
  }

  focus -force .nb.p1.t
  
  updateMessageStatus
}

# Copy, Cut, or Paste the selection from text widget $t into the clipboard.
#
proc copy_text {t} {
  if {![catch {$t get sel.first sel.last} text]} {
     global SELECTION
     selection own -selection PRIMARY -command lose_selection .
     clipboard clear -displayof $t
     clipboard append -displayof $t $text
  }
}
proc cut_text {t} {
  copy_text $t
  catch {$t delete sel.first sel.last}
}
proc paste_text {t} {
  if {![catch {selection get} topaste] ||
      ![catch {selection get -displayof $t -selection CLIPBOARD} topaste]} {
    $t insert insert $topaste
  }
}
proc lose_selection {} {
  global SELECTION
  set SELECTION {}
}
proc retrieve_selection {offset max} {
  global SELECTION
  return [string range $SELECTION $offset [expr {$offset+$max}]]
}
selection handle -selection PRIMARY . retrieve_selection

# Automatically rebuild the catalog display when various search
# parameters change.
#
trace add variable orderby write message_rebuild_catalog_when_idle
trace add variable limit write message_rebuild_catalog_when_idle
trace add variable show write message_rebuild_catalog_when_idle
trace add variable ::showMine write message_rebuild_catalog_when_idle
trace add variable ::showRead write message_rebuild_catalog_when_idle

# The modules of this program can be source-ed in any order.  So when
# this module runs, we do not know if other modules are running yet or
# not.  We assume not.  So anything that depends on another module
# has to go inside the following _start procedure where it will be
# run by an idle callback after all files have been source-ed.
#
proc _start {} {
  rename _start {}
  message_rebuild_catalog_when_idle
  show_catalog

  ::mailroom::user callback after_start
}
after idle _start

proc quit_app {} {
  set g [wm geometry .]
  ::mailroom::config option config geometry -value $g
  ::mailroom::user callback before_exit
  ::mailroom::shutdown
  
  exit 0
}

wm protocol . WM_DELETE_WINDOW {
  quit_app
  destroy .
}
